@@ -5,7 +5,6 @@
 
 #include <math.h>
 
-#include <rte_atomic.h>
 #include <rte_common.h>
 #include <rte_cycles.h>
 #include <rte_debug.h>
@@ -47,10 +46,9 @@ static uint64_t global_bkt_tck_ns;
 static uint64_t global_info_bkt_tck_ns;
 static volatile uint8_t arm_done;
 
-#define CALC_TICKS(tks) ceil((double)((tks) * global_bkt_tck_ns) / global_info_bkt_tck_ns)
+#define CALC_TICKS(tks)					\
+	ceil((double)(tks * global_bkt_tck_ns) / global_info_bkt_tck_ns)
 
-/* Wait double timeout ticks for software and an extra tick for hardware */
-#define WAIT_TICKS(tks) (using_services ? 2 * (tks) : tks + 1)
 
 static bool using_services;
 static uint32_t test_lcore1;
@@ -284,7 +282,7 @@ test_port_conf_cb(uint16_t id, uint8_t event_dev_id, uint8_t *event_port_id,
 }
 
 static int
-_timdev_setup(uint64_t max_tmo_ns, uint64_t bkt_tck_ns)
+_timdev_setup(uint64_t max_tmo_ns, uint64_t bkt_tck_ns, uint64_t flags)
 {
 	struct rte_event_timer_adapter_info info;
 	struct rte_event_timer_adapter_conf config = {
@@ -293,7 +291,7 @@ _timdev_setup(uint64_t max_tmo_ns, uint64_t bkt_tck_ns)
 		.timer_tick_ns = bkt_tck_ns,
 		.max_tmo_ns = max_tmo_ns,
 		.nb_timers = MAX_TIMERS * 10,
-		.flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES,
+		.flags = flags,
 	};
 	uint32_t caps = 0;
 	const char *pool_name = "timdev_test_pool";
@@ -302,6 +300,11 @@ _timdev_setup(uint64_t max_tmo_ns, uint64_t bkt_tck_ns)
 
 	TEST_ASSERT_SUCCESS(rte_event_timer_adapter_caps_get(evdev, &caps),
 				"failed to get adapter capabilities");
+
+	if (flags & RTE_EVENT_TIMER_ADAPTER_F_PERIODIC &&
+	    !(caps & RTE_EVENT_TIMER_ADAPTER_CAP_PERIODIC))
+		return -ENOTSUP;
+
 	if (!(caps & RTE_EVENT_TIMER_ADAPTER_CAP_INTERNAL_PORT)) {
 		timdev = rte_event_timer_adapter_create_ext(&config,
 							    test_port_conf_cb,
@@ -339,42 +342,72 @@ _timdev_setup(uint64_t max_tmo_ns, uint64_t bkt_tck_ns)
 static int
 timdev_setup_usec(void)
 {
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES;
+
 	return using_services ?
 		/* Max timeout is 10,000us and bucket interval is 100us */
-		_timdev_setup(1E7, 1E5) :
+		_timdev_setup(1E7, 1E5, flags) :
 		/* Max timeout is 100us and bucket interval is 1us */
-		_timdev_setup(1E5, 1E3);
+		_timdev_setup(1E5, 1E3, flags);
 }
 
 static int
 timdev_setup_usec_multicore(void)
 {
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES;
+
 	return using_services ?
 		/* Max timeout is 10,000us and bucket interval is 100us */
-		_timdev_setup(1E7, 1E5) :
+		_timdev_setup(1E7, 1E5, flags) :
 		/* Max timeout is 100us and bucket interval is 1us */
-		_timdev_setup(1E5, 1E3);
+		_timdev_setup(1E5, 1E3, flags);
 }
 
 static int
 timdev_setup_msec(void)
 {
-	/* Max timeout is 2 mins, and bucket interval is 100 ms */
-	return _timdev_setup(180 * NSECPERSEC, NSECPERSEC / 10);
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES;
+
+	/* Max timeout is 3 mins, and bucket interval is 100 ms */
+	return _timdev_setup(180 * NSECPERSEC, NSECPERSEC / 10, flags);
+}
+
+static int
+timdev_setup_msec_periodic(void)
+{
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES |
+			 RTE_EVENT_TIMER_ADAPTER_F_PERIODIC;
+
+	/* Periodic mode with 100 ms resolution */
+	return _timdev_setup(0, NSECPERSEC / 10, flags);
 }
 
 static int
 timdev_setup_sec(void)
 {
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES;
+
 	/* Max timeout is 100sec and bucket interval is 1sec */
-	return _timdev_setup(1E11, 1E9);
+	return _timdev_setup(1E11, 1E9, flags);
+}
+
+static int
+timdev_setup_sec_periodic(void)
+{
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES |
+			 RTE_EVENT_TIMER_ADAPTER_F_PERIODIC;
+
+	/* Periodic mode with 1 sec resolution */
+	return _timdev_setup(0, NSECPERSEC, flags);
 }
 
 static int
 timdev_setup_sec_multicore(void)
 {
+	uint64_t flags = RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES;
+
 	/* Max timeout is 100sec and bucket interval is 1sec */
-	return _timdev_setup(1E11, 1E9);
+	return _timdev_setup(1E11, 1E9, flags);
 }
 
 static void
@@ -386,31 +419,10 @@ timdev_teardown(void)
 	rte_mempool_free(eventdev_test_mempool);
 }
 
-static inline uint16_t
-timeout_event_dequeue(struct rte_event *evs, uint64_t nb_evs, uint64_t ticks)
-{
-	uint16_t ev_cnt = 0;
-	uint64_t end_cycle;
-
-	if (using_services && nb_evs == MAX_TIMERS)
-		ticks = 2 * ticks;
-
-	end_cycle = rte_rdtsc() + ticks * global_bkt_tck_ns * rte_get_tsc_hz() / 1E9;
-
-	while (ev_cnt < nb_evs && rte_rdtsc() < end_cycle) {
-		ev_cnt += rte_event_dequeue_burst(evdev, TEST_PORT_ID, &evs[ev_cnt], nb_evs, 0);
-		rte_pause();
-	}
-
-	return ev_cnt;
-}
-
 static inline int
 test_timer_state(void)
 {
 	struct rte_event_timer *ev_tim;
-	const uint64_t max_ticks = 100;
-	uint64_t ticks, wait_ticks;
 	struct rte_event ev;
 	const struct rte_event_timer tim = {
 		.ev.op = RTE_EVENT_OP_NEW,
@@ -421,10 +433,11 @@ test_timer_state(void)
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
 	};
 
+
 	rte_mempool_get(eventdev_test_mempool, (void **)&ev_tim);
 	*ev_tim = tim;
 	ev_tim->ev.event_ptr = ev_tim;
-	ev_tim->timeout_ticks = CALC_TICKS(max_ticks + 20);
+	ev_tim->timeout_ticks = CALC_TICKS(120);
 
 	TEST_ASSERT_EQUAL(rte_event_timer_arm_burst(timdev, &ev_tim, 1), 0,
 			"Armed timer exceeding max_timeout.");
@@ -432,9 +445,8 @@ test_timer_state(void)
 			"Improper timer state set expected %d returned %d",
 			RTE_EVENT_TIMER_ERROR_TOOLATE, ev_tim->state);
 
-	ticks = 10;
 	ev_tim->state = RTE_EVENT_TIMER_NOT_ARMED;
-	ev_tim->timeout_ticks = CALC_TICKS(ticks);
+	ev_tim->timeout_ticks = CALC_TICKS(10);
 
 	TEST_ASSERT_EQUAL(rte_event_timer_arm_burst(timdev, &ev_tim, 1), 1,
 			"Failed to arm timer with proper timeout.");
@@ -443,15 +455,14 @@ test_timer_state(void)
 			RTE_EVENT_TIMER_ARMED, ev_tim->state);
 
 	if (!using_services)
-		wait_ticks = 2 * ticks;
+		rte_delay_us(20);
 	else
-		wait_ticks = ticks;
-
-	TEST_ASSERT_EQUAL(timeout_event_dequeue(&ev, 1, WAIT_TICKS(wait_ticks)), 1,
-			  "Armed timer failed to trigger.");
+		rte_delay_us(1000 + 200);
+	TEST_ASSERT_EQUAL(rte_event_dequeue_burst(evdev, 0, &ev, 1, 0), 1,
+			"Armed timer failed to trigger.");
 
 	ev_tim->state = RTE_EVENT_TIMER_NOT_ARMED;
-	ev_tim->timeout_ticks = CALC_TICKS(max_ticks - 10);
+	ev_tim->timeout_ticks = CALC_TICKS(90);
 	TEST_ASSERT_EQUAL(rte_event_timer_arm_burst(timdev, &ev_tim, 1), 1,
 			"Failed to arm timer with proper timeout.");
 	TEST_ASSERT_EQUAL(rte_event_timer_cancel_burst(timdev, &ev_tim, 1),
@@ -536,6 +547,19 @@ test_timer_arm(void)
 	return TEST_SUCCESS;
 }
 
+static inline int
+test_timer_arm_periodic(void)
+{
+	TEST_ASSERT_SUCCESS(_arm_timers(1, MAX_TIMERS),
+			    "Failed to arm timers");
+	/* With a resolution of 100ms and wait time of 1sec,
+	 * there will be 10 * MAX_TIMERS periodic timer triggers.
+	 */
+	TEST_ASSERT_SUCCESS(_wait_timer_triggers(1, 10 * MAX_TIMERS, 0),
+			    "Timer triggered count doesn't match arm count");
+	return TEST_SUCCESS;
+}
+
 static int
 _arm_wrapper(void *arg)
 {
@@ -611,6 +635,20 @@ test_timer_arm_burst(void)
 	return TEST_SUCCESS;
 }
 
+static inline int
+test_timer_arm_burst_periodic(void)
+{
+	TEST_ASSERT_SUCCESS(_arm_timers_burst(1, MAX_TIMERS),
+			    "Failed to arm timers");
+	/* With a resolution of 100ms and wait time of 1sec,
+	 * there will be 10 * MAX_TIMERS periodic timer triggers.
+	 */
+	TEST_ASSERT_SUCCESS(_wait_timer_triggers(1, 10 * MAX_TIMERS, 0),
+			    "Timer triggered count doesn't match arm count");
+
+	return TEST_SUCCESS;
+}
+
 static int
 _arm_wrapper_burst(void *arg)
 {
@@ -635,6 +673,48 @@ test_timer_arm_burst_multicore(void)
 	return TEST_SUCCESS;
 }
 
+static inline int
+test_timer_cancel_periodic(void)
+{
+	uint64_t i;
+	struct rte_event_timer *ev_tim;
+	const struct rte_event_timer tim = {
+		.ev.op = RTE_EVENT_OP_NEW,
+		.ev.queue_id = 0,
+		.ev.sched_type = RTE_SCHED_TYPE_ATOMIC,
+		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
+		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
+		.state = RTE_EVENT_TIMER_NOT_ARMED,
+		.timeout_ticks = CALC_TICKS(1),
+	};
+
+	for (i = 0; i < MAX_TIMERS; i++) {
+		TEST_ASSERT_SUCCESS(rte_mempool_get(eventdev_test_mempool,
+					(void **)&ev_tim),
+				"mempool alloc failed");
+		*ev_tim = tim;
+		ev_tim->ev.event_ptr = ev_tim;
+
+		TEST_ASSERT_EQUAL(rte_event_timer_arm_burst(timdev, &ev_tim,
+					1), 1, "Failed to arm timer %d",
+				rte_errno);
+
+		rte_delay_us(100 + (i % 5000));
+
+		TEST_ASSERT_EQUAL(rte_event_timer_cancel_burst(timdev,
+					&ev_tim, 1), 1,
+				"Failed to cancel event timer %d", rte_errno);
+		rte_mempool_put(eventdev_test_mempool, ev_tim);
+	}
+
+
+	TEST_ASSERT_SUCCESS(_wait_timer_triggers(30, MAX_TIMERS,
+				MAX_TIMERS),
+		"Timer triggered count doesn't match arm, cancel count");
+
+	return TEST_SUCCESS;
+}
+
 static inline int
 test_timer_cancel(void)
 {
@@ -1047,9 +1127,9 @@ adapter_lookup(void)
 static int
 adapter_start(void)
 {
-	TEST_ASSERT_SUCCESS(_timdev_setup(180 * NSECPERSEC,
-			NSECPERSEC / 10),
-			"Failed to start adapter");
+	TEST_ASSERT_SUCCESS(_timdev_setup(180 * NSECPERSEC, NSECPERSEC / 10,
+					  RTE_EVENT_TIMER_ADAPTER_F_ADJUST_RES),
+			    "Failed to start adapter");
 	TEST_ASSERT_EQUAL(rte_event_timer_adapter_start(timdev), -EALREADY,
 			"Timer adapter started without call to stop.");
 
@@ -1084,9 +1164,8 @@ stat_inc_reset_ev_enq(void)
 	int ret, i, n;
 	int num_evtims = MAX_TIMERS;
 	struct rte_event_timer *evtims[num_evtims];
-	struct rte_event evs[num_evtims];
+	struct rte_event evs[BATCH_SIZE];
 	struct rte_event_timer_adapter_stats stats;
-	uint64_t ticks = 5;
 	const struct rte_event_timer init_tim = {
 		.ev.op = RTE_EVENT_OP_NEW,
 		.ev.queue_id = TEST_QUEUE_ID,
@@ -1094,7 +1173,7 @@ stat_inc_reset_ev_enq(void)
 		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
-		.timeout_ticks = CALC_TICKS(ticks), /**< expire in .5 sec */
+		.timeout_ticks = CALC_TICKS(5), // expire in .5 sec
 	};
 
 	ret = rte_mempool_get_bulk(eventdev_test_mempool, (void **)evtims,
@@ -1119,12 +1198,31 @@ stat_inc_reset_ev_enq(void)
 			  "succeeded = %d, rte_errno = %s",
 			  num_evtims, ret, rte_strerror(rte_errno));
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
-	TEST_ASSERT_EQUAL(n, num_evtims, "Expected %d timer expiry events, got %d",
-			  num_evtims, n);
+	rte_delay_ms(1000);
+
+#define MAX_TRIES num_evtims
+	int sum = 0;
+	int tries = 0;
+	bool done = false;
+	while (!done) {
+		sum += rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs,
+					       RTE_DIM(evs), 10);
+		if (sum >= num_evtims || ++tries >= MAX_TRIES)
+			done = true;
+
+		rte_delay_ms(10);
+	}
+
+	TEST_ASSERT_EQUAL(sum, num_evtims, "Expected %d timer expiry events, "
+			  "got %d", num_evtims, sum);
+
+	TEST_ASSERT(tries < MAX_TRIES, "Exceeded max tries");
+
+	rte_delay_ms(100);
 
 	/* Make sure the eventdev is still empty */
-	n = timeout_event_dequeue(evs, 1, WAIT_TICKS(1));
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs),
+				      10);
 
 	TEST_ASSERT_EQUAL(n, 0, "Dequeued unexpected number of timer expiry "
 			  "events from event device");
@@ -1161,7 +1259,6 @@ event_timer_arm(void)
 	struct rte_event_timer_adapter *adapter = timdev;
 	struct rte_event_timer *evtim = NULL;
 	struct rte_event evs[BATCH_SIZE];
-	uint64_t ticks = 5;
 	const struct rte_event_timer init_tim = {
 		.ev.op = RTE_EVENT_OP_NEW,
 		.ev.queue_id = TEST_QUEUE_ID,
@@ -1169,7 +1266,7 @@ event_timer_arm(void)
 		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
-		.timeout_ticks = CALC_TICKS(ticks), /**< expire in .5 sec */
+		.timeout_ticks = CALC_TICKS(5), // expire in .5 sec
 	};
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
@@ -1196,7 +1293,10 @@ event_timer_arm(void)
 	TEST_ASSERT_EQUAL(rte_errno, EALREADY, "Unexpected rte_errno value "
 			  "after arming already armed timer");
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	/* Let timer expire */
+	rte_delay_ms(1000);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 1, "Failed to dequeue expected number of expiry "
 			  "events from event device");
 
@@ -1216,7 +1316,6 @@ event_timer_arm_double(void)
 	struct rte_event_timer_adapter *adapter = timdev;
 	struct rte_event_timer *evtim = NULL;
 	struct rte_event evs[BATCH_SIZE];
-	uint64_t ticks = 5;
 	const struct rte_event_timer init_tim = {
 		.ev.op = RTE_EVENT_OP_NEW,
 		.ev.queue_id = TEST_QUEUE_ID,
@@ -1224,7 +1323,7 @@ event_timer_arm_double(void)
 		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
-		.timeout_ticks = CALC_TICKS(ticks), /**< expire in .5 sec */
+		.timeout_ticks = CALC_TICKS(5), // expire in .5 sec
 	};
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
@@ -1244,7 +1343,10 @@ event_timer_arm_double(void)
 	TEST_ASSERT_EQUAL(rte_errno, EALREADY, "Unexpected rte_errno value "
 			  "after double-arm");
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	/* Let timer expire */
+	rte_delay_ms(600);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 1, "Dequeued incorrect number of expiry events - "
 			  "expected: 1, actual: %d", n);
 
@@ -1271,7 +1373,6 @@ event_timer_arm_expiry(void)
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
 	};
-	uint64_t ticks = 30;
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
 	if (evtim == NULL) {
@@ -1281,7 +1382,7 @@ event_timer_arm_expiry(void)
 
 	/* Set up an event timer */
 	*evtim = init_tim;
-	evtim->timeout_ticks = CALC_TICKS(ticks); /**< expire in 3 secs */
+	evtim->timeout_ticks = CALC_TICKS(30),	// expire in 3 secs
 	evtim->ev.event_ptr = evtim;
 
 	ret = rte_event_timer_arm_burst(adapter, &evtim, 1);
@@ -1290,10 +1391,17 @@ event_timer_arm_expiry(void)
 	TEST_ASSERT_EQUAL(evtim->state, RTE_EVENT_TIMER_ARMED, "Event "
 			  "timer in incorrect state");
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), ticks - 1);
+	rte_delay_ms(2999);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 0, "Dequeued unexpected timer expiry event");
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(1));
+	/* Delay 100 ms to account for the adapter tick window - should let us
+	 * dequeue one event
+	 */
+	rte_delay_ms(100);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 1, "Dequeued incorrect number (%d) of timer "
 			  "expiry events", n);
 	TEST_ASSERT_EQUAL(evs[0].event_type, RTE_EVENT_TYPE_TIMER,
@@ -1325,7 +1433,6 @@ event_timer_arm_rearm(void)
 		.ev.event_type = RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
 	};
-	uint64_t ticks = 1;
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
 	if (evtim == NULL) {
@@ -1335,7 +1442,7 @@ event_timer_arm_rearm(void)
 
 	/* Set up a timer */
 	*evtim = init_tim;
-	evtim->timeout_ticks = CALC_TICKS(ticks); /**< expire in 0.1 sec */
+	evtim->timeout_ticks = CALC_TICKS(1);  // expire in 0.1 sec
 	evtim->ev.event_ptr = evtim;
 
 	/* Arm it */
@@ -1343,7 +1450,10 @@ event_timer_arm_rearm(void)
 	TEST_ASSERT_EQUAL(ret, 1, "Failed to arm event timer: %s\n",
 			  rte_strerror(rte_errno));
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	/* Add 100ms to account for the adapter tick window */
+	rte_delay_ms(100 + 100);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 1, "Failed to dequeue expected number of expiry "
 			  "events from event device");
 
@@ -1360,7 +1470,10 @@ event_timer_arm_rearm(void)
 	TEST_ASSERT_EQUAL(ret, 1, "Failed to arm event timer: %s\n",
 			  rte_strerror(rte_errno));
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	/* Add 100ms to account for the adapter tick window */
+	rte_delay_ms(100 + 100);
+
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 1, "Failed to dequeue expected number of expiry "
 			  "events from event device");
 
@@ -1382,8 +1495,7 @@ event_timer_arm_max(void)
 	int ret, i, n;
 	int num_evtims = MAX_TIMERS;
 	struct rte_event_timer *evtims[num_evtims];
-	struct rte_event evs[num_evtims];
-	uint64_t ticks = 5;
+	struct rte_event evs[BATCH_SIZE];
 	const struct rte_event_timer init_tim = {
 		.ev.op = RTE_EVENT_OP_NEW,
 		.ev.queue_id = TEST_QUEUE_ID,
@@ -1391,7 +1503,7 @@ event_timer_arm_max(void)
 		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
-		.timeout_ticks = CALC_TICKS(ticks), /**< expire in .5 sec */
+		.timeout_ticks = CALC_TICKS(5), // expire in .5 sec
 	};
 
 	ret = rte_mempool_get_bulk(eventdev_test_mempool, (void **)evtims,
@@ -1411,12 +1523,31 @@ event_timer_arm_max(void)
 			  "succeeded = %d, rte_errno = %s",
 			  num_evtims, ret, rte_strerror(rte_errno));
 
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
-	TEST_ASSERT_EQUAL(n, num_evtims, "Expected %d timer expiry events, got %d",
-			  num_evtims, n);
+	rte_delay_ms(1000);
+
+#define MAX_TRIES num_evtims
+	int sum = 0;
+	int tries = 0;
+	bool done = false;
+	while (!done) {
+		sum += rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs,
+					       RTE_DIM(evs), 10);
+		if (sum >= num_evtims || ++tries >= MAX_TRIES)
+			done = true;
+
+		rte_delay_ms(10);
+	}
+
+	TEST_ASSERT_EQUAL(sum, num_evtims, "Expected %d timer expiry events, "
+			  "got %d", num_evtims, sum);
+
+	TEST_ASSERT(tries < MAX_TRIES, "Exceeded max tries");
+
+	rte_delay_ms(100);
 
 	/* Make sure the eventdev is still empty */
-	n = timeout_event_dequeue(evs, 1, WAIT_TICKS(1));
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs),
+				    10);
 
 	TEST_ASSERT_EQUAL(n, 0, "Dequeued unexpected number of timer expiry "
 			  "events from event device");
@@ -1536,7 +1667,6 @@ event_timer_cancel(void)
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
 	};
-	uint64_t ticks = 30;
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
 	if (evtim == NULL) {
@@ -1554,7 +1684,7 @@ event_timer_cancel(void)
 	/* Set up a timer */
 	*evtim = init_tim;
 	evtim->ev.event_ptr = evtim;
-	evtim->timeout_ticks = CALC_TICKS(ticks); /**< expire in 3 sec */
+	evtim->timeout_ticks = CALC_TICKS(30);  // expire in 3 sec
 
 	/* Check that cancelling an inited but unarmed timer fails */
 	ret = rte_event_timer_cancel_burst(adapter, &evtim, 1);
@@ -1578,8 +1708,10 @@ event_timer_cancel(void)
 	TEST_ASSERT_EQUAL(evtim->state, RTE_EVENT_TIMER_CANCELED,
 			  "evtim in incorrect state");
 
+	rte_delay_ms(3000);
+
 	/* Make sure that no expiry event was generated */
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 0, "Dequeued unexpected timer expiry event\n");
 
 	rte_mempool_put(eventdev_test_mempool, evtim);
@@ -1602,8 +1734,8 @@ event_timer_cancel_double(void)
 		.ev.priority = RTE_EVENT_DEV_PRIORITY_NORMAL,
 		.ev.event_type =  RTE_EVENT_TYPE_TIMER,
 		.state = RTE_EVENT_TIMER_NOT_ARMED,
+		.timeout_ticks = CALC_TICKS(5), // expire in .5 sec
 	};
-	uint64_t ticks = 30;
 
 	rte_mempool_get(eventdev_test_mempool, (void **)&evtim);
 	if (evtim == NULL) {
@@ -1614,7 +1746,7 @@ event_timer_cancel_double(void)
 	/* Set up a timer */
 	*evtim = init_tim;
 	evtim->ev.event_ptr = evtim;
-	evtim->timeout_ticks = CALC_TICKS(ticks); /**< expire in 3 sec */
+	evtim->timeout_ticks = CALC_TICKS(30);  // expire in 3 sec
 
 	ret = rte_event_timer_arm_burst(adapter, &evtim, 1);
 	TEST_ASSERT_EQUAL(ret, 1, "Failed to arm event timer: %s\n",
@@ -1636,8 +1768,10 @@ event_timer_cancel_double(void)
 	TEST_ASSERT_EQUAL(rte_errno, EALREADY, "Unexpected rte_errno value "
 			  "after double-cancel: rte_errno = %d", rte_errno);
 
+	rte_delay_ms(3000);
+
 	/* Still make sure that no expiry event was generated */
-	n = timeout_event_dequeue(evs, RTE_DIM(evs), WAIT_TICKS(ticks));
+	n = rte_event_dequeue_burst(evdev, TEST_PORT_ID, evs, RTE_DIM(evs), 0);
 	TEST_ASSERT_EQUAL(n, 0, "Dequeued unexpected timer expiry event\n");
 
 	rte_mempool_put(eventdev_test_mempool, evtim);
@@ -1751,10 +1885,16 @@ static struct unit_test_suite event_timer_adptr_functional_testsuite  = {
 				test_timer_state),
 		TEST_CASE_ST(timdev_setup_usec, timdev_teardown,
 				test_timer_arm),
+		TEST_CASE_ST(timdev_setup_msec_periodic, timdev_teardown,
+				test_timer_arm_periodic),
 		TEST_CASE_ST(timdev_setup_usec, timdev_teardown,
 				test_timer_arm_burst),
+		TEST_CASE_ST(timdev_setup_msec_periodic, timdev_teardown,
+				test_timer_arm_burst_periodic),
 		TEST_CASE_ST(timdev_setup_sec, timdev_teardown,
 				test_timer_cancel),
+		TEST_CASE_ST(timdev_setup_sec_periodic, timdev_teardown,
+				test_timer_cancel_periodic),
 		TEST_CASE_ST(timdev_setup_sec, timdev_teardown,
 				test_timer_cancel_random),
 		TEST_CASE_ST(timdev_setup_usec_multicore, timdev_teardown,
